@@ -86,11 +86,18 @@ AbstractType::Ptr TypeBuilder::parseType(QString type, AstNode* node)
         }
     } else {
         if (!type.compare(QLatin1String("object"), Qt::CaseInsensitive)) {
-            type = QStringLiteral("stdclass");
+            return AbstractType::Ptr(new IntegralTypeExtended(IntegralTypeExtended::TypeObject));
         }
+
+        QualifiedIdentifier typehint = QualifiedIdentifier(type.toLower().replace(QLatin1Literal("\\"), QLatin1Literal("::")));
+
+        if (typehint.toString().startsWith(QLatin1Literal("::"))) {
+            typehint.setExplicitlyGlobal(true);
+        }
+
         //don't use openTypeFromName as it uses cursor for findDeclarations
-        DeclarationPointer decl = findDeclarationImport(ClassDeclarationType,
-                                                        QualifiedIdentifier(type.toLower()));
+        DeclarationPointer decl = findDeclarationImport(ClassDeclarationType, typehint);
+
         if (decl && decl->abstractType()) {
             return decl->abstractType();
         }
@@ -299,6 +306,7 @@ void TypeBuilder::visitClassStatement(ClassStatementAst *node)
 
         FunctionType::Ptr functionType = FunctionType::Ptr(new FunctionType());
         openType(functionType);
+        openContextType(functionType);
 
         AbstractType::Ptr phpdocReturnType = parseDocComment(node, QStringLiteral("return"));
         functionType->setReturnType(returnType(node->returnType, phpdocReturnType, editor(), currentContext()));
@@ -309,6 +317,7 @@ void TypeBuilder::visitClassStatement(ClassStatementAst *node)
         if (currentType<FunctionType>() && !currentType<FunctionType>()->returnType()) {
             currentType<FunctionType>()->setReturnType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
         }
+        closeContextType();
         closeType();
     } else {
         //member-variable
@@ -324,7 +333,11 @@ void TypeBuilder::visitClassStatement(ClassStatementAst *node)
 void TypeBuilder::visitClassVariable(ClassVariableAst *node)
 {
     if (!m_gotTypeFromDocComment) {
-        openAbstractType(getTypeForNode(node->value));
+        if (node->value) {
+            openAbstractType(getTypeForNode(node->value));
+        } else {
+            openAbstractType(AbstractType::Ptr(new IntegralType(IntegralType::TypeNull)));
+        }
 
         TypeBuilderBase::visitClassVariable(node);
 
@@ -350,6 +363,22 @@ void TypeBuilder::visitConstantDeclaration(ConstantDeclarationAst* node)
     }
 }
 
+void TypeBuilder::visitClassConstantDeclaration(ClassConstantDeclarationAst* node)
+{
+    if (!m_gotTypeFromDocComment || !currentAbstractType()) {
+        AbstractType::Ptr type = getTypeForNode(node->scalar);
+        type->setModifiers(type->modifiers() | AbstractType::ConstModifier);
+        openAbstractType(type);
+
+        TypeBuilderBase::visitClassConstantDeclaration(node);
+
+        closeType();
+    } else {
+        currentAbstractType()->setModifiers(currentAbstractType()->modifiers() & AbstractType::ConstModifier);
+        TypeBuilderBase::visitClassConstantDeclaration(node);
+    }
+}
+
 void TypeBuilder::visitParameter(ParameterAst *node)
 {
     AbstractType::Ptr phpDocTypehint;
@@ -375,6 +404,8 @@ void TypeBuilder::visitFunctionDeclarationStatement(FunctionDeclarationStatement
     FunctionType::Ptr type = currentType<FunctionType>();
     Q_ASSERT(type);
 
+    openContextType(type);
+
     AbstractType::Ptr phpdocReturnType = parseDocComment(node, QStringLiteral("return"));
     type->setReturnType(returnType(node->returnType, phpdocReturnType, editor(), currentContext()));
     m_gotReturnTypeFromDocComment = type->returnType();
@@ -386,6 +417,8 @@ void TypeBuilder::visitFunctionDeclarationStatement(FunctionDeclarationStatement
     if (!type->returnType()) {
         type->setReturnType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
     }
+
+    closeContextType();
 }
 
 void TypeBuilder::visitClosure(ClosureAst* node)
@@ -393,6 +426,7 @@ void TypeBuilder::visitClosure(ClosureAst* node)
     m_currentFunctionParams = parseDocCommentParams(node);
     FunctionType::Ptr type = FunctionType::Ptr(new FunctionType());
     openType(type);
+    openContextType(type);
 
     AbstractType::Ptr phpdocReturnType = parseDocComment(node, QStringLiteral("return"));
     type->setReturnType(returnType(node->returnType, phpdocReturnType, editor(), currentContext()));
@@ -405,6 +439,7 @@ void TypeBuilder::visitClosure(ClosureAst* node)
     if (!type->returnType()) {
         type->setReturnType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
     }
+    closeContextType();
     closeType();
 }
 
@@ -513,7 +548,8 @@ void TypeBuilder::visitStatement(StatementAst* node)
             }
             if (classDec) {
                 /// Qualified identifier for 'iterator'
-                static const QualifiedIdentifier iteratorQId(QStringLiteral("iterator"));
+                static QualifiedIdentifier iteratorQId(QStringLiteral("iterator"));
+                iteratorQId.setExplicitlyGlobal(true);
                 ClassDeclaration* iteratorDecl = dynamic_cast<ClassDeclaration*>(
                     findDeclarationImport(ClassDeclarationType, iteratorQId).data()
                 );
@@ -552,6 +588,28 @@ void TypeBuilder::visitCatchItem(Php::CatchItemAst *node)
 
 }
 
+void TypeBuilder::visitVarExpression(Php::VarExpressionAst *node)
+{
+    if (hasCurrentContextType() && node->isGenerator != -1 && !m_gotReturnTypeFromDocComment) {
+        FunctionType::Ptr ft = FunctionType::Ptr::dynamicCast(currentContextType());
+        static QualifiedIdentifier generatorQId(QStringLiteral("generator"));
+        generatorQId.setExplicitlyGlobal(true);
+        DeclarationPointer generatorDecl = findDeclarationImport(ClassDeclarationType, generatorQId);
+
+        if (ft && generatorDecl) {
+            AbstractType::Ptr generatorType = generatorDecl->abstractType();
+
+            if (generatorType) {
+                ft->setReturnType(generatorType);
+            }
+        }
+
+        updateCurrentType();
+    }
+
+    TypeBuilderBase::visitVarExpression(node);
+}
+
 void TypeBuilder::updateCurrentType()
 {
     // do nothing
